package com.idiot.operationbackend.controller;

import com.idiot.operationbackend.entity.Account;
import com.idiot.operationbackend.entity.AccountFansStat;
import com.idiot.operationbackend.entity.FansActionStat;
import com.idiot.operationbackend.service.facade.AccountFansService;
import com.idiot.operationbackend.service.facade.AccountFansStatService;
import com.idiot.operationbackend.service.facade.AccountService;
import com.idiot.operationbackend.service.facade.FansActionStatService;
import com.idiot.operationbackend.support.Constants;
import com.idiot.operationbackend.support.CustomException;
import com.idiot.operationbackend.support.JsonResult;
import com.idiot.operationbackend.util.JwtTokenUtil;
import com.idiot.operationbackend.vo.StatData;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.http.ResponseEntity;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.idiot.operationbackend.support.Constants.calcRate;

/**
 * 运营星 首页
 * @author wang xiao
 * @date Created in 10:55 2020/9/15
 */
@RestController
@RequestMapping("/index")
@Api(value = "IndexController", tags ="首页")
public class IndexController {


    private final Logger logger = LoggerFactory.getLogger(IndexController.class);

    private final String HOUR = "H";

    private final String WEEK = "W";

    private final String DAY = "D";

    private final String MONTH = "M";

    @Autowired
    private AccountService accountService;

    @Autowired
    private AccountFansStatService fansStatService;

    @Autowired
    private AccountFansService fansService;

    @Autowired
    private FansActionStatService actionStatService;




    @GetMapping("/summary")
    @ApiOperation(value = "查询公众号昨日统计数据概览")
    public ResponseEntity<JsonResult<AccountFansStat>> getFansStat (@RequestHeader String token) {
        String userId = JwtTokenUtil.getUserId(token);
        logger.info("用户:{}查询首页数据统计概览---start",userId);
        List<Account> accounts = accountService.queryAccountByUserId(userId);
        List<String> accountIds =  accounts.stream().map(Account::getId).collect(Collectors.toList());
        // 查询数据  不知道怎么优化
        String yesterdayStr = Constants.DATE_FORMATTER.format(LocalDate.now().plusDays(-1));
        List<AccountFansStat> ydStatData = fansStatService.queryByDateAndAccIds(accountIds,yesterdayStr);
        String beforeYesterdayStr = Constants.DATE_FORMATTER.format(LocalDate.now().plusDays(-2));
        List<AccountFansStat> beforeYdStat = fansStatService.queryByDateAndAccIds(accountIds,beforeYesterdayStr);

        logger.info("用户:{}首页概览数据计算中---ing",userId);
        // 所有公众号昨日数据
        int totalNum = ydStatData.stream().mapToInt(AccountFansStat::getTotalFansNum).sum();
        int addNum = ydStatData.stream().mapToInt(AccountFansStat::getAddNum).sum();
        int cancelNum = ydStatData.stream().mapToInt(AccountFansStat::getCancelNum).sum();
        int newNum = ydStatData.stream().mapToInt(AccountFansStat::getNewNum).sum();
        int inactiveNum = ydStatData.stream().mapToInt(AccountFansStat::getInactiveNum).sum();
        // 所有公众号前日数据
        int bfCancelNum = beforeYdStat.stream().mapToInt(AccountFansStat::getCancelNum).sum();
        int bfAddNum = beforeYdStat.stream().mapToInt(AccountFansStat::getAddNum).sum();
        int bfNewNum = beforeYdStat.stream().mapToInt(AccountFansStat::getNewNum).sum();
        int bfInactiveNum = beforeYdStat.stream().mapToInt(AccountFansStat::getInactiveNum).sum();
        int bfTotalNum = beforeYdStat.stream().mapToInt(AccountFansStat::getTotalFansNum).sum();

        AccountFansStat result = new AccountFansStat();
        result.setAddNum(addNum);
        result.setNewNum(newNum);
        result.setCancelNum(cancelNum);
        result.setInactiveNum(inactiveNum);
        result.setTotalFansNum(totalNum);
        result.setAddRate(calcRate(addNum,bfAddNum));
        result.setCancelRate(calcRate(cancelNum,bfCancelNum));
        result.setNewRate(calcRate(newNum,bfNewNum));
        result.setInactiveRate(calcRate(inactiveNum,bfInactiveNum));
        result.setTotalFansRate(calcRate(totalNum,bfTotalNum));
        logger.info("用户:{}查询首页数据统计概览---end",userId);
        return ResponseEntity.ok(JsonResult.success(result));
    }

    @GetMapping("/single/{accountId}")
    @ApiOperation(value = "查询单个公众号昨日统计数据概览")
    public ResponseEntity<JsonResult<AccountFansStat>> getSingleFansStat (@RequestHeader String token,
                                                                          @PathVariable String accountId) {
        String userId = JwtTokenUtil.getUserId(token);
        logger.info("用户:{}查询首页单个公众号：{}数据统计概览---start",userId,accountId);
        String yesterdayStr = Constants.DATE_FORMATTER.format(LocalDate.now().plusDays(-1));
        AccountFansStat fansStat = fansStatService.queryByDateAndAccountId(accountId,yesterdayStr);
        logger.info("用户:{}查询首页单个公众号：{}数据统计概览---end",userId,accountId);
        return ResponseEntity.ok(JsonResult.success(fansStat));
    }


    @GetMapping("/fansGrowth/{accountId}")
    @ApiOperation(value = "查询单个公众号--粉丝增长")
    public ResponseEntity<JsonResult<Map<String,Object>>> getFansGrowthStat (@PathVariable String accountId,
                                                              @RequestHeader String token,
                                                              @RequestParam String type,
                                                              @RequestParam(required = false) String startDate,
                                                              @RequestParam(required = false) String endDate) {
        String userId = JwtTokenUtil.getUserId(token);
        Map<String,Object> result = new HashMap<>(2);
        logger.info("用户:{}查询首页单个公众号：{}粉丝增长---start",userId,accountId);
        List<StatData> statData = queryFansGrowthStatData(accountId, type, startDate, endDate);
        result.put("tableData",statData);
        logger.info("用户:{}查询首页单个公众号：{}粉丝增长---end",userId,accountId);
        return ResponseEntity.ok(JsonResult.success(result));
    }

    /**
     *  统计粉丝数据  好难受写不出完美的代码
     * @author wangxiao
     * @date 16:00 2020/9/16 
     * @param accountId
     * @param type
     * @param startDate
     * @param endDate
     * @return java.util.List<com.idiot.operationbackend.vo.StatData>
     */
    private List<StatData> queryFansGrowthStatData(String accountId, String type, String startDate, String endDate) {

        int totalNum  = fansService.countFansByAccountId(accountId);
        if (HOUR.equals(type)) {
            // 小时数据 需要在粉丝 动作中统计
            if (!checkDate(startDate,endDate)){
                throw new CustomException(500,"请选择开始和结束时间");
            }

            long start =  LocalDate.parse(startDate).atTime(LocalTime.of(0,0,0))
                    .toInstant(Constants.DEFAULT_ZONE).getEpochSecond();
            long end = LocalDate.parse(endDate).atTime(LocalTime.of(23,59,59))
                    .toInstant(Constants.DEFAULT_ZONE).getEpochSecond();
            // 查询时间段内的值 
            List<FansActionStat> fansActionStats = actionStatService.queryFansActionStat(accountId,start,end);
            if (CollectionUtils.isEmpty(fansActionStats)) {
                throw new CustomException(500,"暂无数据");
            }
            List<StatData> statDataList = new ArrayList<>();
            StatData statData =  null;
            List<FansActionStat> tempStatList = null;
            while (start<=end){
                long finalStart = start;
                // 过滤两小时内
                tempStatList = fansActionStats.stream()
                        .filter(e->e.getCreateTime() >= finalStart && e.getCreateTime()<=  finalStart +7200)
                        .sorted(Comparator.comparingLong(FansActionStat::getCreateTime))
                        .collect(Collectors.toList());
                statData = countStatData(tempStatList,finalStart,finalStart+7200);
                statDataList.add(statData);
                start += 7200;
            }
            return statDataList;
        }
        return null;
    
    }

    private boolean checkDate (String start, String end) {
        return !(StringUtils.isEmpty(start) | StringUtils.isEmpty(end));
    }

    /**
     *  
     * @author wangxiao
     * @date 16:03 2020/9/16 
     * @param fansActionStats
     * @param start
     * @param end
     * @return com.idiot.operationbackend.vo.StatData
     */
    private StatData countStatData (List<FansActionStat> fansActionStats,long start,long end) {

        LocalDateTime startTime =LocalDateTime.ofInstant(Instant.ofEpochSecond(start),Constants.DEFAULT_ZONE);
        LocalDateTime endTime =LocalDateTime.ofInstant(Instant.ofEpochSecond(end),Constants.DEFAULT_ZONE);
        StatData statData = new StatData();
        int newNum = 0;
        int cancelNum = 0;
        int inactiveNum = 0;
        if (!CollectionUtils.isEmpty(fansActionStats)) {
            int size = fansActionStats.size();
            FansActionStat temp = null;
            for (int i = 0; i < size; i++) {
                temp = fansActionStats.get(i);
                switch (temp.getAction()){
                    case Constants.NEW:
                        newNum += 1;
                        break;
                    case Constants.CANCEL:
                        cancelNum += 1;
                        break;
                    default:
                        inactiveNum += 1;
                        break;
                }
            }
        }
        String label = String.format("%s %s:%s-%s:%s",
                startTime.toLocalDate(),startTime.getHour(),"00",endTime.getHour(),"00");
        statData.setDateLabel(label);
        statData.setAddNum(newNum-cancelNum);
        statData.setCancelNum(cancelNum);
        statData.setNewNum(newNum);
        statData.setInactiveNum(inactiveNum);
        return statData;
    }




}
